@@ -29,7 +29,7 @@
 #define _GNU_SOURCE /* Stupid GNU :-P */
 
 #define PACKAGE "BitlBee"
-#define BITLBEE_VERSION "1.2"
+#define BITLBEE_VERSION "1.2.1"
 #define VERSION BITLBEE_VERSION
 
 #define MAX_STRING 511
@@ -94,10 +94,6 @@
 #undef g_main_quit
 #define g_main_quit		__PLEASE_USE_B_MAIN_QUIT__
 
-#ifndef F_OK
-#define F_OK 0
-#endif
-
 #ifndef G_GNUC_MALLOC
 /* Doesn't exist in GLib <=2.4 while everything else in BitlBee should
    work with it, so let's fake this one. */
@@ -159,6 +155,8 @@ void root_command_string( irc_t *irc, user_t *u, char *command, int flags );
 void root_command( irc_t *irc, char *command[] );
 gboolean bitlbee_shutdown( gpointer data, gint fd, b_input_condition cond );
 
+char *set_eval_root_nick( set_t *set, char *new_nick );
+
 extern global_t global;
 
 #endif
