@@ -44,7 +44,8 @@ conf_t *conf_load( int argc, char *argv[] )
 	
 	conf = g_new0( conf_t, 1 );
 	
-	conf->iface = NULL;
+	conf->iface_in = NULL;
+	conf->iface_out = NULL;
 	conf->port = g_strdup( "6667" );
 	conf->nofork = 0;
 	conf->verbose = 0;
@@ -81,7 +82,7 @@ conf_t *conf_load( int argc, char *argv[] )
 	{
 		if( opt == 'i' )
 		{
-			conf->iface = g_strdup( optarg );
+			conf->iface_in = g_strdup( optarg );
 		}
 		else if( opt == 'p' )
 		{
@@ -130,7 +131,7 @@ conf_t *conf_load( int argc, char *argv[] )
 			        "An IRC-to-other-chat-networks gateway\n"
 			        "\n"
 			        "  -I  Classic/InetD mode. (Default)\n"
-			        "  -D  Daemon mode. (Still EXPERIMENTAL!)\n"
+			        "  -D  Daemon mode. (one process serves all)\n"
 			        "  -F  Forking daemon. (one process per client)\n"
 				"  -u  Run daemon as specified user.\n"
 			        "  -P  Specify PID-file (not for inetd mode)\n"
@@ -201,14 +202,19 @@ static int conf_loadini( conf_t *conf, char *file )
 			}
 			else if( g_strcasecmp( ini->key, "daemoninterface" ) == 0 )
 			{
-				g_free( conf->iface );
-				conf->iface = g_strdup( ini->value );
+				g_free( conf->iface_in );
+				conf->iface_in = g_strdup( ini->value );
 			}
 			else if( g_strcasecmp( ini->key, "daemonport" ) == 0 )
 			{
 				g_free( conf->port );
 				conf->port = g_strdup( ini->value );
 			}
+			else if( g_strcasecmp( ini->key, "clientinterface" ) == 0 )
+			{
+				g_free( conf->iface_out );
+				conf->iface_out = g_strdup( ini->value );
+			}
 			else if( g_strcasecmp( ini->key, "authmode" ) == 0 )
 			{
 				if( g_strcasecmp( ini->value, "registered" ) == 0 )
@@ -251,7 +257,7 @@ static int conf_loadini( conf_t *conf, char *file )
 			else if( g_strcasecmp( ini->key, "account_storage_migrate" ) == 0 )
 			{
 				g_strfreev( conf->migrate_storage );
-				conf->migrate_storage = g_strsplit( ini->value, " \t,;", -1 );
+				conf->migrate_storage = g_strsplit_set( ini->value, " \t,;", -1 );
 			}
 			else if( g_strcasecmp( ini->key, "pinginterval" ) == 0 )
 			{
