@@ -26,43 +26,52 @@
 #include "nogaim.h"
 #include "msn.h"
 
-static struct prpl *my_protocol = NULL;
+static char *msn_set_display_name( set_t *set, char *value );
 
-static void msn_login( struct aim_user *acct )
+static void msn_init( account_t *acc )
 {
-	struct gaim_connection *gc = new_gaim_conn( acct );
+	set_t *s;
+	
+	s = set_add( &acc->set, "display_name", NULL, msn_set_display_name, acc );
+	s->flags |= ACC_SET_NOSAVE | ACC_SET_ONLINE_ONLY;
+
+	s = set_add( &acc->set, "mail_notifications", "false", set_eval_bool, acc );
+}
+
+static void msn_login( account_t *acc )
+{
+	struct im_connection *ic = imcb_new( acc );
 	struct msn_data *md = g_new0( struct msn_data, 1 );
 	
-	set_login_progress( gc, 1, "Connecting" );
-	
-	gc->proto_data = md;
+	ic->proto_data = md;
 	md->fd = -1;
 	
-	if( strchr( acct->username, '@' ) == NULL )
+	if( strchr( acc->user, '@' ) == NULL )
 	{
-		hide_login_progress( gc, "Invalid account name" );
-		signoff( gc );
+		imcb_error( ic, "Invalid account name" );
+		imc_logout( ic, FALSE );
 		return;
 	}
 	
-	md->fd = proxy_connect( "messenger.hotmail.com", 1863, msn_ns_connected, gc );
+	imcb_log( ic, "Connecting" );
+	
+	md->fd = proxy_connect( "messenger.hotmail.com", 1863, msn_ns_connected, ic );
 	if( md->fd < 0 )
 	{
-		hide_login_progress( gc, "Could not connect to server" );
-		signoff( gc );
-	}
-	else
-	{
-		md->gc = gc;
-		md->away_state = msn_away_state_list;
-		
-		msn_connections = g_slist_append( msn_connections, gc );
+		imcb_error( ic, "Could not connect to server" );
+		imc_logout( ic, TRUE );
+		return;
 	}
+	
+	md->ic = ic;
+	md->away_state = msn_away_state_list;
+	
+	msn_connections = g_slist_append( msn_connections, ic );
 }
 
-static void msn_close( struct gaim_connection *gc )
+static void msn_logout( struct im_connection *ic )
 {
-	struct msn_data *md = gc->proto_data;
+	struct msn_data *md = ic->proto_data;
 	GSList *l;
 	
 	if( md )
@@ -80,42 +89,32 @@ static void msn_close( struct gaim_connection *gc )
 		while( md->switchboards )
 			msn_sb_destroy( md->switchboards->data );
 		
-		if( md->msgq )
-		{
-			struct msn_message *m;
-			
-			for( l = md->msgq; l; l = l->next )
-			{
-				m = l->data;
-			
-				serv_got_crap( gc, "Warning: Closing down MSN connection with unsent message to %s, you'll have to resend it.", m->who );
-				g_free( m->who );
-				g_free( m->text );
-				g_free( m );
-			}
-			g_slist_free( md->msgq );
-		}
+		msn_msgq_purge( ic, &md->msgq );
+		
+		while( md->groupcount > 0 )
+			g_free( md->grouplist[--md->groupcount] );
+		g_free( md->grouplist );
 		
 		g_free( md );
 	}
 	
-	for( l = gc->permit; l; l = l->next )
+	for( l = ic->permit; l; l = l->next )
 		g_free( l->data );
-	g_slist_free( gc->permit );
+	g_slist_free( ic->permit );
 	
-	for( l = gc->deny; l; l = l->next )
+	for( l = ic->deny; l; l = l->next )
 		g_free( l->data );
-	g_slist_free( gc->deny );
+	g_slist_free( ic->deny );
 	
-	msn_connections = g_slist_remove( msn_connections, gc );
+	msn_connections = g_slist_remove( msn_connections, ic );
 }
 
-static int msn_send_im( struct gaim_connection *gc, char *who, char *message, int len, int away )
+static int msn_buddy_msg( struct im_connection *ic, char *who, char *message, int away )
 {
 	struct msn_switchboard *sb;
-	struct msn_data *md = gc->proto_data;
+	struct msn_data *md = ic->proto_data;
 	
-	if( ( sb = msn_sb_by_handle( gc, who ) ) )
+	if( ( sb = msn_sb_by_handle( ic, who ) ) )
 	{
 		return( msn_sb_sendmessage( sb, message ) );
 	}
@@ -130,7 +129,7 @@ static int msn_send_im( struct gaim_connection *gc, char *who, char *message, in
 		m->text = g_strdup( message );
 		
 		/* FIXME: *CHECK* the reliability of using spare sb's! */
-		if( ( sb = msn_sb_spare( gc ) ) )
+		if( ( sb = msn_sb_spare( ic ) ) )
 		{
 			debug( "Trying to use a spare switchboard to message %s", who );
 			
@@ -148,7 +147,7 @@ static int msn_send_im( struct gaim_connection *gc, char *who, char *message, in
 		
 		/* If we reach this line, there was no spare switchboard, so let's make one. */
 		g_snprintf( buf, sizeof( buf ), "XFR %d SB\r\n", ++md->trId );
-		if( !msn_write( gc, buf, strlen( buf ) ) )
+		if( !msn_write( ic, buf, strlen( buf ) ) )
 		{
 			g_free( m->who );
 			g_free( m->text );
@@ -168,31 +167,22 @@ static int msn_send_im( struct gaim_connection *gc, char *who, char *message, in
 	return( 0 );
 }
 
-static GList *msn_away_states( struct gaim_connection *gc )
+static GList *msn_away_states( struct im_connection *ic )
 {
-	GList *l = NULL;
+	static GList *l = NULL;
 	int i;
 	
-	for( i = 0; msn_away_state_list[i].number > -1; i ++ )
-		l = g_list_append( l, (void*) msn_away_state_list[i].name );
+	if( l == NULL )
+		for( i = 0; msn_away_state_list[i].number > -1; i ++ )
+			l = g_list_append( l, (void*) msn_away_state_list[i].name );
 	
-	return( l );
+	return l;
 }
 
-static char *msn_get_status_string( struct gaim_connection *gc, int number )
-{
-	const struct msn_away_state *st = msn_away_state_by_number( number );
-	
-	if( st )
-		return( (char*) st->name );
-	else
-		return( "" );
-}
-
-static void msn_set_away( struct gaim_connection *gc, char *state, char *message )
+static void msn_set_away( struct im_connection *ic, char *state, char *message )
 {
 	char buf[1024];
-	struct msn_data *md = gc->proto_data;
+	struct msn_data *md = ic->proto_data;
 	const struct msn_away_state *st;
 	
 	if( strcmp( state, GAIM_AWAY_CUSTOM ) == 0 )
@@ -204,72 +194,43 @@ static void msn_set_away( struct gaim_connection *gc, char *state, char *message
 	md->away_state = st;
 	
 	g_snprintf( buf, sizeof( buf ), "CHG %d %s\r\n", ++md->trId, st->code );
-	msn_write( gc, buf, strlen( buf ) );
+	msn_write( ic, buf, strlen( buf ) );
 }
 
-static void msn_set_info( struct gaim_connection *gc, char *info )
+static void msn_set_my_name( struct im_connection *ic, char *info )
 {
-	int i;
-	char buf[1024], *fn, *s;
-	struct msn_data *md = gc->proto_data;
-	
-	if( strlen( info ) > 129 )
-	{
-		do_error_dialog( gc, "Maximum name length exceeded", "MSN" );
-		return;
-	}
-	
-	/* Of course we could use http_encode() here, but when we encode
-	   every character, the server is less likely to complain about the
-	   chosen name. However, the MSN server doesn't seem to like escaped
-	   non-ASCII chars, so we keep those unescaped. */
-	s = fn = g_new0( char, strlen( info ) * 3 + 1 );
-	for( i = 0; info[i]; i ++ )
-		if( info[i] & 128 )
-		{
-			*s = info[i];
-			s ++;
-		}
-		else
-		{
-			g_snprintf( s, 4, "%%%02X", info[i] );
-			s += 3;
-		}
-	
-	g_snprintf( buf, sizeof( buf ), "REA %d %s %s\r\n", ++md->trId, gc->username, fn );
-	msn_write( gc, buf, strlen( buf ) );
-	g_free( fn );
+	msn_set_display_name( set_find( &ic->acc->set, "display_name" ), info );
 }
 
-static void msn_get_info(struct gaim_connection *gc, char *who) 
+static void msn_get_info(struct im_connection *ic, char *who) 
 {
 	/* Just make an URL and let the user fetch the info */
-	serv_got_crap( gc, "%s\n%s: %s%s", _("User Info"), _("For now, fetch yourself"), PROFILE_URL, who );
+	imcb_log( ic, "%s\n%s: %s%s", _("User Info"), _("For now, fetch yourself"), PROFILE_URL, who );
 }
 
-static void msn_add_buddy( struct gaim_connection *gc, char *who )
+static void msn_add_buddy( struct im_connection *ic, char *who, char *group )
 {
-	msn_buddy_list_add( gc, "FL", who, who );
+	msn_buddy_list_add( ic, "FL", who, who );
 }
 
-static void msn_remove_buddy( struct gaim_connection *gc, char *who, char *group )
+static void msn_remove_buddy( struct im_connection *ic, char *who, char *group )
 {
-	msn_buddy_list_remove( gc, "FL", who );
+	msn_buddy_list_remove( ic, "FL", who );
 }
 
-static int msn_chat_send( struct gaim_connection *gc, int id, char *message )
+static void msn_chat_msg( struct groupchat *c, char *message, int flags )
 {
-	struct msn_switchboard *sb = msn_sb_by_id( gc, id );
+	struct msn_switchboard *sb = msn_sb_by_chat( c );
 	
 	if( sb )
-		return( msn_sb_sendmessage( sb, message ) );
-	else
-		return( 0 );
+		msn_sb_sendmessage( sb, message );
+	/* FIXME: Error handling (although this can't happen unless something's
+	   already severely broken) disappeared here! */
 }
 
-static void msn_chat_invite( struct gaim_connection *gc, int id, char *msg, char *who )
+static void msn_chat_invite( struct groupchat *c, char *who, char *message )
 {
-	struct msn_switchboard *sb = msn_sb_by_id( gc, id );
+	struct msn_switchboard *sb = msn_sb_by_chat( c );
 	char buf[1024];
 	
 	if( sb )
@@ -279,39 +240,35 @@ static void msn_chat_invite( struct gaim_connection *gc, int id, char *msg, char
 	}
 }
 
-static void msn_chat_leave( struct gaim_connection *gc, int id )
+static void msn_chat_leave( struct groupchat *c )
 {
-	struct msn_switchboard *sb = msn_sb_by_id( gc, id );
+	struct msn_switchboard *sb = msn_sb_by_chat( c );
 	
 	if( sb )
 		msn_sb_write( sb, "OUT\r\n", 5 );
 }
 
-static int msn_chat_open( struct gaim_connection *gc, char *who )
+static struct groupchat *msn_chat_with( struct im_connection *ic, char *who )
 {
 	struct msn_switchboard *sb;
-	struct msn_data *md = gc->proto_data;
+	struct msn_data *md = ic->proto_data;
 	char buf[1024];
 	
-	if( ( sb = msn_sb_by_handle( gc, who ) ) )
+	if( ( sb = msn_sb_by_handle( ic, who ) ) )
 	{
 		debug( "Converting existing switchboard to %s to a groupchat", who );
-		msn_sb_to_chat( sb );
-		return( 1 );
+		return msn_sb_to_chat( sb );
 	}
 	else
 	{
 		struct msn_message *m;
 		
-		if( ( sb = msn_sb_spare( gc ) ) )
+		if( ( sb = msn_sb_spare( ic ) ) )
 		{
 			debug( "Trying to reuse an existing switchboard as a groupchat with %s", who );
 			g_snprintf( buf, sizeof( buf ), "CAL %d %s\r\n", ++sb->trId, who );
 			if( msn_sb_write( sb, buf, strlen( buf ) ) )
-			{
-				msn_sb_to_chat( sb );
-				return( 1 );
-			}
+				return msn_sb_to_chat( sb );
 		}
 		
 		/* If the stuff above failed for some reason: */
@@ -319,7 +276,7 @@ static int msn_chat_open( struct gaim_connection *gc, char *who )
 		
 		/* Request a new switchboard. */
 		g_snprintf( buf, sizeof( buf ), "XFR %d SB\r\n", ++md->trId );
-		if( !msn_write( gc, buf, strlen( buf ) ) )
+		if( !msn_write( ic, buf, strlen( buf ) ) )
 			return( 0 );
 		
 		/* Create a magic message. This is quite hackish, but who cares? :-P */
@@ -330,77 +287,111 @@ static int msn_chat_open( struct gaim_connection *gc, char *who )
 		/* Queue the magic message and cross your fingers. */
 		md->msgq = g_slist_append( md->msgq, m );
 		
-		return( 1 );
+		/* FIXME: Can I try to return something here already? */
+		return NULL;
 	}
 	
-	return( 0 );
+	return NULL;
 }
 
-static void msn_keepalive( struct gaim_connection *gc )
+static void msn_keepalive( struct im_connection *ic )
 {
-	msn_write( gc, "PNG\r\n", strlen( "PNG\r\n" ) );
+	msn_write( ic, "PNG\r\n", strlen( "PNG\r\n" ) );
 }
 
-static void msn_add_permit( struct gaim_connection *gc, char *who )
+static void msn_add_permit( struct im_connection *ic, char *who )
 {
-	msn_buddy_list_add( gc, "AL", who, who );
+	msn_buddy_list_add( ic, "AL", who, who );
 }
 
-static void msn_rem_permit( struct gaim_connection *gc, char *who )
+static void msn_rem_permit( struct im_connection *ic, char *who )
 {
-	msn_buddy_list_remove( gc, "AL", who );
+	msn_buddy_list_remove( ic, "AL", who );
 }
 
-static void msn_add_deny( struct gaim_connection *gc, char *who )
+static void msn_add_deny( struct im_connection *ic, char *who )
 {
 	struct msn_switchboard *sb;
 	
-	msn_buddy_list_add( gc, "BL", who, who );
+	msn_buddy_list_add( ic, "BL", who, who );
 	
 	/* If there's still a conversation with this person, close it. */
-	if( ( sb = msn_sb_by_handle( gc, who ) ) )
+	if( ( sb = msn_sb_by_handle( ic, who ) ) )
 	{
 		msn_sb_destroy( sb );
 	}
 }
 
-static void msn_rem_deny( struct gaim_connection *gc, char *who )
+static void msn_rem_deny( struct im_connection *ic, char *who )
 {
-	msn_buddy_list_remove( gc, "BL", who );
+	msn_buddy_list_remove( ic, "BL", who );
 }
 
-static int msn_send_typing( struct gaim_connection *gc, char *who, int typing )
+static int msn_send_typing( struct im_connection *ic, char *who, int typing )
 {
-	if( typing )
-		return( msn_send_im( gc, who, TYPING_NOTIFICATION_MESSAGE, strlen( TYPING_NOTIFICATION_MESSAGE ), 0 ) );
+	if( typing & OPT_TYPING )
+		return( msn_buddy_msg( ic, who, TYPING_NOTIFICATION_MESSAGE, 0 ) );
 	else
 		return( 1 );
 }
 
-void msn_init(struct prpl *ret)
+static char *msn_set_display_name( set_t *set, char *value )
 {
-	ret->protocol = PROTO_MSN;
+	account_t *acc = set->data;
+	struct im_connection *ic = acc->ic;
+	struct msn_data *md;
+	char buf[1024], *fn;
+	
+	/* Double-check. */
+	if( ic == NULL )
+		return NULL;
+	
+	md = ic->proto_data;
+	
+	if( strlen( value ) > 129 )
+	{
+		imcb_log( ic, "Maximum name length exceeded" );
+		return NULL;
+	}
+	
+	fn = msn_http_encode( value );
+	
+	g_snprintf( buf, sizeof( buf ), "REA %d %s %s\r\n", ++md->trId, ic->acc->user, fn );
+	msn_write( ic, buf, strlen( buf ) );
+	g_free( fn );
+	
+	/* Returning NULL would be better, because the server still has to
+	   confirm the name change. However, it looks a bit confusing to the
+	   user. */
+	return value;
+}
+
+void msn_initmodule()
+{
+	struct prpl *ret = g_new0(struct prpl, 1);
+	
+	ret->name = "msn";
 	ret->login = msn_login;
-	ret->close = msn_close;
-	ret->send_im = msn_send_im;
+	ret->init = msn_init;
+	ret->logout = msn_logout;
+	ret->buddy_msg = msn_buddy_msg;
 	ret->away_states = msn_away_states;
-	ret->get_status_string = msn_get_status_string;
 	ret->set_away = msn_set_away;
-	ret->set_info = msn_set_info;
 	ret->get_info = msn_get_info;
+	ret->set_my_name = msn_set_my_name;
 	ret->add_buddy = msn_add_buddy;
 	ret->remove_buddy = msn_remove_buddy;
-	ret->chat_send = msn_chat_send;
+	ret->chat_msg = msn_chat_msg;
 	ret->chat_invite = msn_chat_invite;
 	ret->chat_leave = msn_chat_leave;
-	ret->chat_open = msn_chat_open;
+	ret->chat_with = msn_chat_with;
 	ret->keepalive = msn_keepalive;
 	ret->add_permit = msn_add_permit;
 	ret->rem_permit = msn_rem_permit;
 	ret->add_deny = msn_add_deny;
 	ret->rem_deny = msn_rem_deny;
 	ret->send_typing = msn_send_typing;
-	ret->cmp_buddynames = g_strcasecmp;
+	ret->handle_cmp = g_strcasecmp;
 
-	my_protocol = ret;
+	register_protocol(ret);
 }
