@@ -1,3 +1,6 @@
+#include <errno.h>
+#include <fcntl.h>
+
 #ifndef _WIN32
 #include <unistd.h>
 #include <sys/socket.h>
@@ -5,8 +8,11 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 #define sock_make_nonblocking(fd) fcntl(fd, F_SETFL, O_NONBLOCK)
+#define sock_make_blocking(fd) fcntl(fd, F_SETFL, 0)
 #define sockerr_again() (errno == EINPROGRESS || errno == EINTR)
+#ifndef EVENTS_LIBEVENT
 #define closesocket(a) close(a)
+#endif
 #else
 # include <winsock2.h>
 # ifndef _MSC_VER
@@ -21,6 +27,7 @@
 # define umask _umask
 # define mode_t int
 # define sock_make_nonblocking(fd) { int non_block = 1; ioctlsocket(fd, FIONBIO, &non_block); }
+# define sock_make_blocking(fd) { int non_block = 0; ioctlsocket(fd, FIONBIO, &non_block); }
 # define sockerr_again() (WSAGetLastError() == WSAEINTR || WSAGetLastError() == WSAEINPROGRESS || WSAGetLastError() == WSAEWOULDBLOCK)
 # define ETIMEDOUT WSAETIMEDOUT
 # define sleep(a) Sleep(a*1000)
